"\#{%(As of version 0.10.2 BoxGrinder no longer *mounts* any ephemeral disks by \ndefault for S3 AMIs.).bold}\n\nIt is possible to modify device mappings at build-time if you desire by using:\n \#{%(--delivery-config block_device_mappings:\"/dev/sdb=ephemeral0&/dev/sdc=ephemeral1\").bold}\n\nYou may specify additional EBS devices to be created and attached at launch-time, \nsee documentation for examples.\n\nAlternatively, mappings can be specified at launch-time rather than build-time.\n \nFor fuller details, including an outline of terminology and different strategies\nfor attaching and mounting, see the following resource: \n \#{%(http://www.boxgrinder.org/permalink/ephemeral#s3).bold}\nDO\n"