"\#{%(As of version 0.10.2 BoxGrinder no longer *attaches* or *mounts* any \nephemeral disks by default for EBS AMIs.).bold}\n\nIt is still possible to specify device mappings at build-time if you desire by\nusing: \n \#{%(--delivery-config block_device_mappings:\"/dev/sdb=ephemeral0&/dev/sdc=ephemeral1\").bold}\n\nYou may specify additional EBS devices to be created and attached at launch-time, \nsee documentation for examples.\n\nAlternatively, mappings can be specified at launch-time rather than build-time.\n \nFor fuller details, including an outline of terminology and different strategies\nfor attaching and mounting, see the following resource: \n \#{%(http://www.boxgrinder.org/permalink/ephemeral#ebs).bold}\nDO\n"